@@ -139,8 +139,6 @@ svn_error_t *svn_fs_bdb__set_uuid(svn_fs_t *fs,
   value.data = apr_pstrmemdup(pool, uuid, value.size + 1);
 
   svn_fs_base__trail_debug(trail, "uuids", "put");
-  SVN_ERR(BDB_WRAP(fs, _("set repository uuid"),
-                   uuids->put(uuids, trail->db_txn, &key, &value, 0)));
-
-  return SVN_NO_ERROR;
+  return BDB_WRAP(fs, _("set repository uuid"),
+                  uuids->put(uuids, trail->db_txn, &key, &value, 0));
 }
