@@ -15,7 +15,6 @@
  * ====================================================================
  */
 
-#include <assert.h>
 #include <string.h>
 
 #include <apr_tables.h>
@@ -162,7 +161,7 @@ svn_fs_base__youngest_rev(svn_revnum_t *youngest_p,
   svn_revnum_t youngest;
   SVN_ERR(svn_fs__check_fs(fs, TRUE));
   SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_youngest_rev, &youngest,
-                                 pool));
+                                 TRUE, pool));
   *youngest_p = youngest;
   return SVN_NO_ERROR;
 }
@@ -200,7 +199,7 @@ svn_fs_base__revision_proplist(apr_hash_t **table_p,
   args.table_p = &table;
   args.rev = rev;
   SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_revision_proplist, &args,
-                                 pool));
+                                 FALSE, pool));
 
   *table_p = table ? table : apr_hash_make(pool);
   return SVN_NO_ERROR;
@@ -223,7 +222,7 @@ svn_fs_base__revision_prop(svn_string_t **value_p,
   args.table_p = &table;
   args.rev = rev;
   SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_revision_proplist, &args,
-                                 pool));
+                                 FALSE, pool));
 
   /* And then the prop from that list (if there was a list). */
   *value_p = NULL;
@@ -275,11 +274,9 @@ txn_body_change_rev_prop(void *baton, trail_t *trail)
 {
   struct change_rev_prop_args *args = baton;
 
-  SVN_ERR(svn_fs_base__set_rev_prop(trail->fs, args->rev,
-                                    args->name, args->value,
-                                    trail, trail->pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_base__set_rev_prop(trail->fs, args->rev,
+                                   args->name, args->value,
+                                   trail, trail->pool);
 }
 
 
@@ -297,9 +294,8 @@ svn_fs_base__change_rev_prop(svn_fs_t *fs,
   args.rev = rev;
   args.name = name;
   args.value = value;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_change_rev_prop, &args, pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_base__retry_txn(fs, txn_body_change_rev_prop, &args, 
+                                TRUE, pool);
 }
 
 
@@ -315,8 +311,7 @@ svn_fs_base__txn_make_committed(svn_fs_t *fs,
 {
   transaction_t *txn;
 
-  /* Don't you dare call this with an invalid REVISION. */
-  assert(SVN_IS_VALID_REVNUM(revision));
+  SVN_ERR_ASSERT(SVN_IS_VALID_REVNUM(revision));
 
   /* Make sure the TXN is not committed already. */
   SVN_ERR(get_txn(&txn, fs, txn_name, FALSE, trail, pool));
@@ -491,7 +486,8 @@ svn_fs_base__txn_proplist(apr_hash_t **table_p,
 
   args.table_p = &table;
   args.id = txn->id;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_txn_proplist, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_txn_proplist, &args,
+                                 FALSE, pool));
 
   *table_p = table ? table : apr_hash_make(pool);
   return SVN_NO_ERROR;
@@ -513,7 +509,8 @@ svn_fs_base__txn_prop(svn_string_t **value_p,
   /* Get the proplist. */
   args.table_p = &table;
   args.id = txn->id;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_txn_proplist, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_txn_proplist, &args,
+                                 FALSE, pool));
 
   /* And then the prop from that list (if there was a list). */
   *value_p = NULL;
@@ -585,9 +582,8 @@ svn_fs_base__change_txn_prop(svn_fs_txn_t *txn,
   args.id = txn->id;
   args.name = name;
   args.value = value;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_change_txn_prop, &args, pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_base__retry_txn(fs, txn_body_change_txn_prop, &args,
+                                TRUE, pool);
 }
 
 
@@ -605,7 +601,7 @@ svn_fs_base__change_txn_props(svn_fs_txn_t *txn,
 
       svn_pool_clear(iterpool);
 
-      SVN_ERR(svn_fs_base__change_txn_prop(txn, prop->name, 
+      SVN_ERR(svn_fs_base__change_txn_prop(txn, prop->name,
                                            prop->value, iterpool));
     }
   svn_pool_destroy(iterpool);
@@ -714,7 +710,7 @@ svn_fs_base__begin_txn(svn_fs_txn_t **txn_p,
   args.txn_p = &txn;
   args.rev   = rev;
   args.flags = flags;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_begin_txn, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_begin_txn, &args, FALSE, pool));
 
   *txn_p = txn;
 
@@ -725,10 +721,8 @@ svn_fs_base__begin_txn(svn_fs_txn_t **txn_p,
      automatically overwritten with a revision datestamp. */
   date.data = svn_time_to_cstring(apr_time_now(), pool);
   date.len = strlen(date.data);
-  SVN_ERR(svn_fs_base__change_txn_prop(txn, SVN_PROP_REVISION_DATE,
-                                       &date, pool));
-
-  return SVN_NO_ERROR;
+  return svn_fs_base__change_txn_prop(txn, SVN_PROP_REVISION_DATE,
+                                       &date, pool);
 }
 
 
@@ -773,7 +767,7 @@ svn_fs_base__open_txn(svn_fs_txn_t **txn_p,
 
   args.txn_p = &txn;
   args.name = name;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_open_txn, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_open_txn, &args, FALSE, pool));
 
   *txn_p = txn;
   return SVN_NO_ERROR;
@@ -890,7 +884,8 @@ delete_txn_tree(svn_fs_t *fs,
   dirent_args.dirents = &dirents;
   dirent_args.id = id;
   dirent_args.txn_id = txn_id;
-  err = svn_fs_base__retry_txn(fs, txn_body_get_dirents, &dirent_args, pool);
+  err = svn_fs_base__retry_txn(fs, txn_body_get_dirents, &dirent_args,
+                               FALSE, pool);
   if (err && (err->apr_err == SVN_ERR_FS_ID_NOT_FOUND))
     {
       svn_error_clear(err);
@@ -920,7 +915,8 @@ delete_txn_tree(svn_fs_t *fs,
   /* Remove the node. */
   rm_args.id = id;
   rm_args.txn_id = txn_id;
-  return svn_fs_base__retry_txn(fs, txn_body_remove_node, &rm_args, pool);
+  return svn_fs_base__retry_txn(fs, txn_body_remove_node, &rm_args,
+                                TRUE, pool);
 }
 
 
@@ -945,7 +941,8 @@ svn_fs_base__purge_txn(svn_fs_t *fs,
   /* Open the transaction, expecting it to be dead. */
   args.txn_p = &txn;
   args.name = txn_id;
-  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_cleanup_txn, &args, pool));
+  SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_cleanup_txn, &args,
+                                 FALSE, pool));
 
   /* Delete the mutable portion of the tree hanging from the
      transaction (which should gracefully recover if we've already
@@ -955,7 +952,7 @@ svn_fs_base__purge_txn(svn_fs_t *fs,
   /* Kill the transaction's changes (which should gracefully recover
      if...). */
   SVN_ERR(svn_fs_base__retry_txn(fs, txn_body_cleanup_txn_changes,
-                                 (void *)txn_id, pool));
+                                 (void *)txn_id, TRUE, pool));
 
   /* Kill the transaction's copies (which should gracefully...). */
   if (txn->copies)
@@ -965,14 +962,14 @@ svn_fs_base__purge_txn(svn_fs_t *fs,
           SVN_ERR(svn_fs_base__retry_txn
                   (fs, txn_body_cleanup_txn_copy,
                    (void *)APR_ARRAY_IDX(txn->copies, i, const char *),
-                   pool));
+                   TRUE, pool));
         }
     }
 
   /* Kill the transaction itself (which ... just kidding -- this has
      no graceful failure mode). */
   return svn_fs_base__retry_txn(fs, txn_body_delete_txn, (void *)txn_id,
-                                pool);
+                                TRUE, pool);
 }
 
 
@@ -1000,7 +997,8 @@ svn_fs_base__abort_txn(svn_fs_txn_t *txn,
   SVN_ERR(svn_fs__check_fs(txn->fs, TRUE));
 
   /* Set the transaction to "dead". */
-  SVN_ERR(svn_fs_base__retry_txn(txn->fs, txn_body_abort_txn, txn, pool));
+  SVN_ERR(svn_fs_base__retry_txn(txn->fs, txn_body_abort_txn, txn,
+                                 TRUE, pool));
 
   /* Now, purge it. */
   SVN_ERR_W(svn_fs_base__purge_txn(txn->fs, txn->id, pool),
@@ -1036,7 +1034,8 @@ svn_fs_base__list_transactions(apr_array_header_t **names_p,
 
   args.names_p = &names;
   args.pool = pool;
-  SVN_ERR(svn_fs_base__retry(fs, txn_body_list_transactions, &args, pool));
+  SVN_ERR(svn_fs_base__retry(fs, txn_body_list_transactions, &args,
+                             FALSE, pool));
 
   *names_p = names;
   return SVN_NO_ERROR;
