@@ -1,8 +1,8 @@
 /*
- * editorp.c :  Pipelined variation of the ra_svn editor
+ * editorp.c :  Driving and consuming an editor across an svn connection
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006, 2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -21,11 +21,7 @@
 #define APR_WANT_STRFUNC
 #include <apr_want.h>
 #include <apr_general.h>
-#include <apr_lib.h>
 #include <apr_strings.h>
-#include <apr_md5.h>
-
-#include <assert.h>
 
 #include "svn_types.h"
 #include "svn_string.h"
@@ -34,7 +30,7 @@
 #include "svn_delta.h"
 #include "svn_ra_svn.h"
 #include "svn_pools.h"
-#include "svn_md5.h"
+#include "svn_private_config.h"
 
 #include "ra_svn.h"
 
@@ -72,6 +68,7 @@ typedef struct {
   apr_pool_t *pool;
   apr_pool_t *file_pool;
   int file_refs;
+  svn_boolean_t for_replay;
 } ra_svn_driver_state_t;
 
 /* Works for both directories and files; however, the pool handling is
@@ -118,7 +115,7 @@ static ra_svn_baton_t *ra_svn_make_baton(svn_ra_svn_conn_t *conn,
  * get one, abort the edit and return the error. */
 static svn_error_t *check_for_error(ra_svn_edit_baton_t *eb, apr_pool_t *pool)
 {
-  assert(!eb->got_status);
+  SVN_ERR_ASSERT(!eb->got_status);
   if (svn_ra_svn__input_waiting(eb->conn, pool))
     {
       eb->got_status = TRUE;
@@ -126,7 +123,7 @@ static svn_error_t *check_for_error(ra_svn_edit_baton_t *eb, apr_pool_t *pool)
       SVN_ERR(svn_ra_svn_read_cmd_response(eb->conn, pool, ""));
       /* We shouldn't get here if the consumer is doing its job. */
       return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                              "Successful edit status returned too soon");
+                              _("Successful edit status returned too soon"));
     }
   return SVN_NO_ERROR;
 }
@@ -173,8 +170,8 @@ static svn_error_t *ra_svn_add_dir(const char *path, void *parent_baton,
   ra_svn_baton_t *b = parent_baton;
   const char *token = make_token('d', b->eb, pool);
 
-  assert((copy_path && SVN_IS_VALID_REVNUM(copy_rev))
-         || (!copy_path && !SVN_IS_VALID_REVNUM(copy_rev)));
+  SVN_ERR_ASSERT((copy_path && SVN_IS_VALID_REVNUM(copy_rev))
+                 || (!copy_path && !SVN_IS_VALID_REVNUM(copy_rev)));
   SVN_ERR(check_for_error(b->eb, pool));
   SVN_ERR(svn_ra_svn_write_cmd(b->conn, pool, "add-dir", "ccc(?cr)", path,
                                b->token, token, copy_path, copy_rev));
@@ -217,6 +214,22 @@ static svn_error_t *ra_svn_close_dir(void *dir_baton, apr_pool_t *pool)
   return SVN_NO_ERROR;
 }
 
+static svn_error_t *ra_svn_absent_dir(const char *path,
+                                      void *parent_baton, apr_pool_t *pool)
+{
+  ra_svn_baton_t *b = parent_baton;
+
+  /* Avoid sending an unknown command if the other end doesn't support
+     absent-dir. */
+  if (! svn_ra_svn_has_capability(b->conn, SVN_RA_SVN_CAP_ABSENT_ENTRIES))
+    return SVN_NO_ERROR;
+
+  SVN_ERR(check_for_error(b->eb, pool));
+  SVN_ERR(svn_ra_svn_write_cmd(b->conn, pool, "absent-dir", "cc", path,
+                               b->token));
+  return SVN_NO_ERROR;
+}
+
 static svn_error_t *ra_svn_add_file(const char *path,
                                     void *parent_baton,
                                     const char *copy_path,
@@ -227,8 +240,8 @@ static svn_error_t *ra_svn_add_file(const char *path,
   ra_svn_baton_t *b = parent_baton;
   const char *token = make_token('c', b->eb, pool);
 
-  assert((copy_path && SVN_IS_VALID_REVNUM(copy_rev))
-         || (!copy_path && !SVN_IS_VALID_REVNUM(copy_rev)));
+  SVN_ERR_ASSERT((copy_path && SVN_IS_VALID_REVNUM(copy_rev))
+                 || (!copy_path && !SVN_IS_VALID_REVNUM(copy_rev)));
   SVN_ERR(check_for_error(b->eb, pool));
   SVN_ERR(svn_ra_svn_write_cmd(b->conn, pool, "add-file", "ccc(?cr)", path,
                                b->token, token, copy_path, copy_rev));
@@ -295,10 +308,13 @@ static svn_error_t *ra_svn_apply_textdelta(void *file_baton,
   diff_stream = svn_stream_create(b, pool);
   svn_stream_set_write(diff_stream, ra_svn_svndiff_handler);
   svn_stream_set_close(diff_stream, ra_svn_svndiff_close_handler);
-  svn_txdelta_to_svndiff(diff_stream, pool, wh, wh_baton);
+  if (svn_ra_svn_has_capability(b->conn, SVN_RA_SVN_CAP_SVNDIFF1))
+    svn_txdelta_to_svndiff2(wh, wh_baton, diff_stream, 1, pool);
+  else
+    svn_txdelta_to_svndiff2(wh, wh_baton, diff_stream, 0, pool);
   return SVN_NO_ERROR;
 }
-  
+
 static svn_error_t *ra_svn_change_file_prop(void *file_baton,
                                             const char *name,
                                             const svn_string_t *value,
@@ -324,12 +340,28 @@ static svn_error_t *ra_svn_close_file(void *file_baton,
   return SVN_NO_ERROR;
 }
 
+static svn_error_t *ra_svn_absent_file(const char *path,
+                                       void *parent_baton, apr_pool_t *pool)
+{
+  ra_svn_baton_t *b = parent_baton;
+
+  /* Avoid sending an unknown command if the other end doesn't support
+     absent-file. */
+  if (! svn_ra_svn_has_capability(b->conn, SVN_RA_SVN_CAP_ABSENT_ENTRIES))
+    return SVN_NO_ERROR;
+
+  SVN_ERR(check_for_error(b->eb, pool));
+  SVN_ERR(svn_ra_svn_write_cmd(b->conn, pool, "absent-file", "cc", path,
+                               b->token));
+  return SVN_NO_ERROR;
+}
+
 static svn_error_t *ra_svn_close_edit(void *edit_baton, apr_pool_t *pool)
 {
   ra_svn_edit_baton_t *eb = edit_baton;
   svn_error_t *err;
 
-  assert(!eb->got_status);
+  SVN_ERR_ASSERT(!eb->got_status);
   eb->got_status = TRUE;
   SVN_ERR(svn_ra_svn_write_cmd(eb->conn, pool, "close-edit", ""));
   err = svn_ra_svn_read_cmd_response(eb->conn, pool, "");
@@ -354,11 +386,11 @@ static svn_error_t *ra_svn_abort_edit(void *edit_baton, apr_pool_t *pool)
   return SVN_NO_ERROR;
 }
 
-void svn_ra_svn__get_editorp(const svn_delta_editor_t **editor,
-                             void **edit_baton, svn_ra_svn_conn_t *conn,
-                             apr_pool_t *pool,
-                             svn_ra_svn_edit_callback callback,
-                             void *callback_baton)
+void svn_ra_svn_get_editor(const svn_delta_editor_t **editor,
+                           void **edit_baton, svn_ra_svn_conn_t *conn,
+                           apr_pool_t *pool,
+                           svn_ra_svn_edit_callback callback,
+                           void *callback_baton)
 {
   svn_delta_editor_t *ra_svn_editor = svn_delta_default_editor(pool);
   ra_svn_edit_baton_t *eb;
@@ -377,11 +409,13 @@ void svn_ra_svn__get_editorp(const svn_delta_editor_t **editor,
   ra_svn_editor->open_directory = ra_svn_open_dir;
   ra_svn_editor->change_dir_prop = ra_svn_change_dir_prop;
   ra_svn_editor->close_directory = ra_svn_close_dir;
+  ra_svn_editor->absent_directory = ra_svn_absent_dir;
   ra_svn_editor->add_file = ra_svn_add_file;
   ra_svn_editor->open_file = ra_svn_open_file;
   ra_svn_editor->apply_textdelta = ra_svn_apply_textdelta;
   ra_svn_editor->change_file_prop = ra_svn_change_file_prop;
   ra_svn_editor->close_file = ra_svn_close_file;
+  ra_svn_editor->absent_file = ra_svn_absent_file;
   ra_svn_editor->close_edit = ra_svn_close_edit;
   ra_svn_editor->abort_edit = ra_svn_abort_edit;
 
@@ -416,7 +450,7 @@ static svn_error_t *lookup_token(ra_svn_driver_state_t *ds, const char *token,
   *entry = apr_hash_get(ds->tokens, token, APR_HASH_KEY_STRING);
   if (!*entry || (*entry)->is_file != is_file)
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "Invalid file or dir token during edit");
+                            _("Invalid file or dir token during edit"));
   return SVN_NO_ERROR;
 }
 
@@ -544,7 +578,25 @@ static svn_error_t *ra_svn_handle_close_dir(svn_ra_svn_conn_t *conn,
   /* Close the directory and destroy the baton. */
   SVN_CMD_ERR(ds->editor->close_directory(entry->baton, pool));
   apr_hash_set(ds->tokens, token, APR_HASH_KEY_STRING, NULL);
-  apr_pool_destroy(entry->pool);
+  svn_pool_destroy(entry->pool);
+  return SVN_NO_ERROR;
+}
+
+static svn_error_t *ra_svn_handle_absent_dir(svn_ra_svn_conn_t *conn,
+                                             apr_pool_t *pool,
+                                             apr_array_header_t *params,
+                                             ra_svn_driver_state_t *ds)
+{
+  const char *path;
+  const char *token;
+  ra_svn_token_entry_t *entry;
+
+  /* Parse parameters and look up the directory token. */
+  SVN_ERR(svn_ra_svn_parse_tuple(params, pool, "cc", &path, &token));
+  SVN_ERR(lookup_token(ds, token, FALSE, &entry));
+
+  /* Call the editor. */
+  SVN_CMD_ERR(ds->editor->absent_directory(path, entry->baton, pool));
   return SVN_NO_ERROR;
 }
 
@@ -607,7 +659,7 @@ static svn_error_t *ra_svn_handle_apply_textdelta(svn_ra_svn_conn_t *conn,
   SVN_ERR(lookup_token(ds, token, TRUE, &entry));
   if (entry->dstream)
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "Apply-textdelta already active");
+                            _("Apply-textdelta already active"));
   entry->pool = svn_pool_create(ds->file_pool);
   SVN_CMD_ERR(ds->editor->apply_textdelta(entry->baton, base_checksum,
                                           entry->pool, &wh, &wh_baton));
@@ -629,7 +681,7 @@ static svn_error_t *ra_svn_handle_textdelta_chunk(svn_ra_svn_conn_t *conn,
   SVN_ERR(lookup_token(ds, token, TRUE, &entry));
   if (!entry->dstream)
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "Apply-textdelta not active");
+                            _("Apply-textdelta not active"));
   SVN_CMD_ERR(svn_stream_write(entry->dstream, str->data, &str->len));
   return SVN_NO_ERROR;
 }
@@ -647,10 +699,10 @@ static svn_error_t *ra_svn_handle_textdelta_end(svn_ra_svn_conn_t *conn,
   SVN_ERR(lookup_token(ds, token, TRUE, &entry));
   if (!entry->dstream)
     return svn_error_create(SVN_ERR_RA_SVN_MALFORMED_DATA, NULL,
-                            "Apply-textdelta not active");
+                            _("Apply-textdelta not active"));
   SVN_CMD_ERR(svn_stream_close(entry->dstream));
   entry->dstream = NULL;
-  apr_pool_destroy(entry->pool);
+  svn_pool_destroy(entry->pool);
   return SVN_NO_ERROR;
 }
 
@@ -688,7 +740,25 @@ static svn_error_t *ra_svn_handle_close_file(svn_ra_svn_conn_t *conn,
   SVN_CMD_ERR(ds->editor->close_file(entry->baton, text_checksum, pool));
   apr_hash_set(ds->tokens, token, APR_HASH_KEY_STRING, NULL);
   if (--ds->file_refs == 0)
-    apr_pool_clear(ds->file_pool);
+    svn_pool_clear(ds->file_pool);
+  return SVN_NO_ERROR;
+}
+
+static svn_error_t *ra_svn_handle_absent_file(svn_ra_svn_conn_t *conn,
+                                              apr_pool_t *pool,
+                                              apr_array_header_t *params,
+                                              ra_svn_driver_state_t *ds)
+{
+  const char *path;
+  const char *token;
+  ra_svn_token_entry_t *entry;
+
+  /* Parse parameters and look up the parent directory token. */
+  SVN_ERR(svn_ra_svn_parse_tuple(params, pool, "cc", &path, &token));
+  SVN_ERR(lookup_token(ds, token, FALSE, &entry));
+
+  /* Call the editor. */
+  SVN_CMD_ERR(ds->editor->absent_file(path, entry->baton, pool));
   return SVN_NO_ERROR;
 }
 
@@ -716,6 +786,21 @@ static svn_error_t *ra_svn_handle_abort_edit(svn_ra_svn_conn_t *conn,
   return svn_ra_svn_write_cmd_response(conn, pool, "");
 }
 
+static svn_error_t *ra_svn_handle_finish_replay(svn_ra_svn_conn_t *conn,
+                                                apr_pool_t *pool,
+                                                apr_array_header_t *params,
+                                                ra_svn_driver_state_t *ds)
+{
+  if (!ds->for_replay)
+    return svn_error_createf
+      (SVN_ERR_RA_SVN_UNKNOWN_CMD, NULL,
+       _("Command 'finish-replay' invalid outside of replays"));
+  ds->done = TRUE;
+  if (ds->aborted)
+    *ds->aborted = FALSE;
+  return SVN_NO_ERROR;
+}
+
 static const struct {
   const char *cmd;
   svn_error_t *(*handler)(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
@@ -729,6 +814,7 @@ static const struct {
   { "open-dir",         ra_svn_handle_open_dir },
   { "change-dir-prop",  ra_svn_handle_change_dir_prop },
   { "close-dir",        ra_svn_handle_close_dir },
+  { "absent-dir",       ra_svn_handle_absent_dir },
   { "add-file",         ra_svn_handle_add_file },
   { "open-file",        ra_svn_handle_open_file },
   { "apply-textdelta",  ra_svn_handle_apply_textdelta },
@@ -736,8 +822,10 @@ static const struct {
   { "textdelta-end",    ra_svn_handle_textdelta_end },
   { "change-file-prop", ra_svn_handle_change_file_prop },
   { "close-file",       ra_svn_handle_close_file },
+  { "absent-file",      ra_svn_handle_absent_file },
   { "close-edit",       ra_svn_handle_close_edit },
   { "abort-edit",       ra_svn_handle_abort_edit },
+  { "finish-replay",    ra_svn_handle_finish_replay },
   { NULL }
 };
 
@@ -759,11 +847,12 @@ static svn_error_t *blocked_write(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
   return SVN_NO_ERROR;
 }
 
-svn_error_t *svn_ra_svn__drive_editorp(svn_ra_svn_conn_t *conn,
-                                       apr_pool_t *pool,
-                                       const svn_delta_editor_t *editor,
-                                       void *edit_baton,
-                                       svn_boolean_t *aborted)
+svn_error_t *svn_ra_svn_drive_editor2(svn_ra_svn_conn_t *conn,
+                                      apr_pool_t *pool,
+                                      const svn_delta_editor_t *editor,
+                                      void *edit_baton,
+                                      svn_boolean_t *aborted,
+                                      svn_boolean_t for_replay)
 {
   ra_svn_driver_state_t state;
   apr_pool_t *subpool = svn_pool_create(pool);
@@ -780,10 +869,11 @@ svn_error_t *svn_ra_svn__drive_editorp(svn_ra_svn_conn_t *conn,
   state.pool = pool;
   state.file_pool = svn_pool_create(pool);
   state.file_refs = 0;
+  state.for_replay = for_replay;
 
   while (!state.done)
     {
-      apr_pool_clear(subpool);
+      svn_pool_clear(subpool);
       SVN_ERR(svn_ra_svn_read_tuple(conn, subpool, "wl", &cmd, &params));
       for (i = 0; ra_svn_edit_cmds[i].cmd; i++)
         {
@@ -795,7 +885,7 @@ svn_error_t *svn_ra_svn__drive_editorp(svn_ra_svn_conn_t *conn,
       else
         {
           err = svn_error_createf(SVN_ERR_RA_SVN_UNKNOWN_CMD, NULL,
-                                  "Unknown command '%s'", cmd);
+                                  _("Unknown command '%s'"), cmd);
           err = svn_error_create(SVN_ERR_RA_SVN_CMD_ERR, err, NULL);
         }
 
@@ -806,10 +896,12 @@ svn_error_t *svn_ra_svn__drive_editorp(svn_ra_svn_conn_t *conn,
           if (!state.done)
             {
               /* Abort the edit and use non-blocking I/O to write the error. */
-              svn_error_clear (editor->abort_edit(edit_baton, subpool));
+              svn_error_clear(editor->abort_edit(edit_baton, subpool));
               svn_ra_svn__set_block_handler(conn, blocked_write, &state);
             }
-          write_err = svn_ra_svn_write_cmd_failure(conn, subpool, err->child);
+          write_err = svn_ra_svn_write_cmd_failure(
+                          conn, subpool,
+                          svn_ra_svn__locate_real_error_child(err));
           if (!write_err)
             write_err = svn_ra_svn_flush(conn, subpool);
           svn_ra_svn__set_block_handler(conn, NULL, NULL);
@@ -820,14 +912,51 @@ svn_error_t *svn_ra_svn__drive_editorp(svn_ra_svn_conn_t *conn,
       SVN_ERR(err);
     }
 
-  /* Read and discard editing commands until the edit is complete. */
+  /* Read and discard editing commands until the edit is complete.
+     Hopefully, the other side will call another editor command, run
+     check_for_error, notice the error, write "abort-edit" at us, and
+     throw the error up a few levels on its side (possibly even
+     tossing it right back at us, which is why we can return
+     SVN_NO_ERROR below).
+
+     However, if the other side is way ahead of us, it might
+     completely finish the edit (or sequence of edit/revprops, for
+     "replay-range") before we send over our "failure".  So we should
+     also stop if we see "success".  (Then the other side will try to
+     interpret our "failure" as a command, which will itself fail...
+     The net effect is that whatever error we wrote to the other side
+     will be replaced with SVN_ERR_RA_SVN_UNKNOWN_CMD.)
+   */
   while (!state.done)
     {
-      apr_pool_clear(subpool);
-      SVN_ERR(svn_ra_svn_read_tuple(conn, subpool, "wl", &cmd, &params));
-      state.done = (strcmp(cmd, "abort-edit") == 0);
+      svn_pool_clear(subpool);
+      err = svn_ra_svn_read_tuple(conn, subpool, "wl", &cmd, &params);
+      if (err && err->apr_err == SVN_ERR_RA_SVN_CONNECTION_CLOSED)
+        {
+          /* Other side disconnected; that's no error. */
+          svn_error_clear(err);
+          svn_pool_destroy(subpool);
+          return SVN_NO_ERROR;
+        }
+      svn_error_clear(err);
+      if (strcmp(cmd, "abort-edit") == 0
+          || strcmp(cmd, "success") == 0)
+        state.done = TRUE;
     }
 
-  apr_pool_destroy(subpool);
+  svn_pool_destroy(subpool);
   return SVN_NO_ERROR;
 }
+
+svn_error_t *svn_ra_svn_drive_editor(svn_ra_svn_conn_t *conn, apr_pool_t *pool,
+                                     const svn_delta_editor_t *editor,
+                                     void *edit_baton,
+                                     svn_boolean_t *aborted)
+{
+  return svn_ra_svn_drive_editor2(conn,
+                                  pool,
+                                  editor,
+                                  edit_baton,
+                                  aborted,
+                                  FALSE);
+}
