@@ -33,8 +33,6 @@
 #include "svn_mergeinfo.h"
 #include "repos.h"
 
-#include <assert.h>
-
 
 /* Note:  this binary search assumes that the datestamp properties on
    each revision are in chronological order.  That is if revision A >
@@ -66,9 +64,7 @@ get_time(apr_time_t *tm,
       (SVN_ERR_FS_GENERAL, NULL,
        _("Failed to find time on revision %ld"), rev);
 
-  SVN_ERR(svn_time_from_cstring(tm, date_str->data, pool));
-
-  return SVN_NO_ERROR;
+  return svn_time_from_cstring(tm, date_str->data, pool);
 }
 
 
@@ -177,25 +173,6 @@ svn_repos_get_committed_info(svn_revnum_t *committed_rev,
   return SVN_NO_ERROR;
 }
 
-
-/* Deprecated. */
-svn_error_t *
-svn_repos_history(svn_fs_t *fs,
-                  const char *path,
-                  svn_repos_history_func_t history_func,
-                  void *history_baton,
-                  svn_revnum_t start,
-                  svn_revnum_t end,
-                  svn_boolean_t cross_copies,
-                  apr_pool_t *pool)
-{
-  return svn_repos_history2(fs, path, history_func, history_baton,
-                            NULL, NULL,
-                            start, end, cross_copies, pool);
-}
-
-
-
 svn_error_t *
 svn_repos_history2(svn_fs_t *fs,
                    const char *path,
@@ -590,7 +567,7 @@ check_ancestry_of_peg_path(svn_boolean_t *is_ancestor,
   /* We must have had at least one iteration above where we
      reassigned fs_path. Else, the path wouldn't have existed at
      future_revision and svn_fs_history would have thrown. */
-  assert(fs_path != NULL);
+  SVN_ERR_ASSERT(fs_path != NULL);
 
   *is_ancestor = (history && strcmp(path, fs_path) == 0);
 
@@ -672,8 +649,7 @@ svn_repos_trace_node_locations(svn_fs_t *fs,
   apr_pool_t *lastpool, *currpool;
   const svn_fs_id_t *id;
 
-  /* Sanity check. */
-  assert(location_revisions_orig->elt_size == sizeof(svn_revnum_t));
+  SVN_ERR_ASSERT(location_revisions_orig->elt_size == sizeof(svn_revnum_t));
 
   /* Ensure that FS_PATH is absolute, because our path-math below will
      depend on that being the case.  */
@@ -886,8 +862,8 @@ svn_repos_node_location_segments(svn_repos_t *repos,
 
   /* Are the revision properly ordered?  They better be -- the API
      demands it. */
-  assert(end_rev <= start_rev);
-  assert(start_rev <= peg_revision);
+  SVN_ERR_ASSERT(end_rev <= start_rev);
+  SVN_ERR_ASSERT(start_rev <= peg_revision);
 
   /* Ensure that PATH is absolute, because our path-math will depend
      on that being the case.  */
@@ -920,6 +896,7 @@ svn_repos_node_location_segments(svn_repos_t *repos,
       segment = apr_pcalloc(subpool, sizeof(*segment));
       segment->range_end = current_rev;
       segment->range_start = end_rev;
+      /* segment path should be absolute without leading '/'. */
       segment->path = cur_path + 1;
 
       SVN_ERR(svn_repos__prev_location(&appeared_rev, &prev_path, &prev_rev,
@@ -953,9 +930,13 @@ svn_repos_node_location_segments(svn_repos_t *repos,
           svn_boolean_t readable;
           svn_fs_root_t *cur_rev_root;
 
+          /* authz_read_func requires path to have a leading slash. */
+          const char *abs_path = apr_pstrcat(subpool, "/", segment->path,
+                                             NULL);
+
           SVN_ERR(svn_fs_revision_root(&cur_rev_root, fs,
                                        segment->range_end, subpool));
-          SVN_ERR(authz_read_func(&readable, cur_rev_root, segment->path,
+          SVN_ERR(authz_read_func(&readable, cur_rev_root, abs_path,
                                   authz_read_baton, subpool));
           if (! readable)
             return SVN_NO_ERROR;
@@ -1006,7 +987,7 @@ get_path_mergeinfo(apr_hash_t **mergeinfo,
 
   SVN_ERR(svn_fs_revision_root(&root, fs, revnum, subpool));
   /* We do not need to call svn_repos_fs_get_mergeinfo() (which performs authz)
-     because we will filter out unreadable revisions in 
+     because we will filter out unreadable revisions in
      find_interesting_revision(), above */
   SVN_ERR(svn_fs_get_mergeinfo(&tmp_catalog, root, paths,
                                svn_mergeinfo_inherited, FALSE, subpool));
@@ -1048,7 +1029,8 @@ struct path_revision
 };
 
 /* Check for merges in OLD_PATH_REV->PATH at OLD_PATH_REV->REVNUM.  Store
-   the mergeinfo difference in MERGED_MERGEINFO, allocated in POOL. */
+   the mergeinfo difference in *MERGED_MERGEINFO, allocated in POOL.  The
+   returned *MERGED_MERGEINFO will be NULL if there are no changes. */
 static svn_error_t *
 get_merged_mergeinfo(apr_hash_t **merged_mergeinfo,
                      svn_repos_t *repos,
@@ -1058,6 +1040,30 @@ get_merged_mergeinfo(apr_hash_t **merged_mergeinfo,
   apr_pool_t *subpool = svn_pool_create(pool);
   apr_hash_t *curr_mergeinfo, *prev_mergeinfo, *deleted, *changed;
   svn_error_t *err;
+  svn_fs_root_t *root;
+  apr_hash_t *changed_paths;
+  const char *path = old_path_rev->path;
+
+  /* Getting/parsing/diffing svn:mergeinfo is expensive, so only do it
+     if there is a property change. */
+  SVN_ERR(svn_fs_revision_root(&root, repos->fs, old_path_rev->revnum,
+                               subpool));
+  SVN_ERR(svn_fs_paths_changed2(&changed_paths, root, subpool));
+  while (1)
+    {
+      svn_fs_path_change2_t *changed_path = apr_hash_get(changed_paths,
+                                                         path,
+                                                         APR_HASH_KEY_STRING);
+      if (changed_path && changed_path->prop_mod)
+        break;
+      if (svn_dirent_is_root(path, strlen(path)))
+        {
+          svn_pool_destroy(subpool);
+          *merged_mergeinfo = NULL;
+          return SVN_NO_ERROR;
+        }
+      path = svn_path_dirname(path, subpool);
+    }
 
   /* First, find the mergeinfo difference for old_path_rev->revnum, and
      old_path_rev->revnum - 1. */
@@ -1067,10 +1073,12 @@ get_merged_mergeinfo(apr_hash_t **merged_mergeinfo,
                            old_path_rev->revnum - 1, subpool);
   if (err && err->apr_err == SVN_ERR_FS_NOT_FOUND)
     {
-      /* If the path doesn't exist in the previous revision, assume empty
-         mergeinfo. */
+      /* If the path doesn't exist in the previous revision, assume no
+         merges */
       svn_error_clear(err);
-      prev_mergeinfo = apr_hash_make(subpool);
+      svn_pool_destroy(subpool);
+      *merged_mergeinfo = NULL;
+      return SVN_NO_ERROR;
     }
   else
     SVN_ERR(err);
@@ -1078,10 +1086,14 @@ get_merged_mergeinfo(apr_hash_t **merged_mergeinfo,
   /* Then calculate and merge the differences. */
   SVN_ERR(svn_mergeinfo_diff(&deleted, &changed, prev_mergeinfo, curr_mergeinfo,
                              FALSE, subpool));
-  SVN_ERR(svn_mergeinfo_merge(changed, deleted, subpool));
+  if (apr_hash_count(deleted))
+    SVN_ERR(svn_mergeinfo_merge(changed, deleted, subpool));
 
   /* Store the result. */
-  *merged_mergeinfo = svn_mergeinfo_dup(changed, pool);
+  if (apr_hash_count(changed))
+    *merged_mergeinfo = svn_mergeinfo_dup(changed, pool);
+  else
+    *merged_mergeinfo = NULL;
 
   svn_pool_destroy(subpool);
 
@@ -1099,47 +1111,49 @@ find_interesting_revisions(apr_array_header_t *path_revisions,
                            apr_hash_t *duplicate_path_revs,
                            svn_repos_authz_func_t authz_read_func,
                            void *authz_read_baton,
-                           apr_pool_t *pool)
+                           apr_pool_t *result_pool,
+                           apr_pool_t *scratch_pool)
 {
-  apr_pool_t *iter_pool, *last_pool;
+  apr_pool_t *iterpool, *last_pool;
   svn_fs_history_t *history;
   svn_fs_root_t *root;
   svn_node_kind_t kind;
 
-  /* We switch betwwen two pools while looping, since we need information from
+  /* We switch between two pools while looping, since we need information from
      the last iteration to be available. */
-  iter_pool = svn_pool_create(pool);
-  last_pool = svn_pool_create(pool);
+  iterpool = svn_pool_create(scratch_pool);
+  last_pool = svn_pool_create(scratch_pool);
 
   /* The path had better be a file in this revision. */
-  SVN_ERR(svn_fs_revision_root(&root, repos->fs, end, last_pool));
-  SVN_ERR(svn_fs_check_path(&kind, root, path, pool));
+  SVN_ERR(svn_fs_revision_root(&root, repos->fs, end, scratch_pool));
+  SVN_ERR(svn_fs_check_path(&kind, root, path, scratch_pool));
   if (kind != svn_node_file)
     return svn_error_createf
       (SVN_ERR_FS_NOT_FILE, NULL, _("'%s' is not a file in revision %ld"),
        path, end);
 
   /* Open a history object. */
-  SVN_ERR(svn_fs_node_history(&history, root, path, last_pool));
-
+  SVN_ERR(svn_fs_node_history(&history, root, path, scratch_pool));
   while (1)
     {
-      struct path_revision *path_rev = apr_palloc(pool, sizeof(*path_rev));
+      struct path_revision *path_rev;
+      svn_revnum_t tmp_revnum;
+      const char *tmp_path;
       apr_pool_t *tmp_pool;
 
-      svn_pool_clear(iter_pool);
+      svn_pool_clear(iterpool);
 
       /* Fetch the history object to walk through. */
-      SVN_ERR(svn_fs_history_prev(&history, history, TRUE, iter_pool));
+      SVN_ERR(svn_fs_history_prev(&history, history, TRUE, iterpool));
       if (!history)
         break;
-      SVN_ERR(svn_fs_history_location(&path_rev->path, &path_rev->revnum,
-                                      history, iter_pool));
+      SVN_ERR(svn_fs_history_location(&tmp_path, &tmp_revnum,
+                                      history, iterpool));
 
       /* Check to see if we already saw this path (and it's ancestors) */
       if (include_merged_revisions
-          && is_path_in_hash(duplicate_path_revs, path_rev->path,
-                             path_rev->revnum, iter_pool))
+          && is_path_in_hash(duplicate_path_revs, tmp_path,
+                             tmp_revnum, iterpool))
          break;
 
       /* Check authorization. */
@@ -1148,21 +1162,24 @@ find_interesting_revisions(apr_array_header_t *path_revisions,
           svn_boolean_t readable;
           svn_fs_root_t *tmp_root;
 
-          SVN_ERR(svn_fs_revision_root(&tmp_root, repos->fs, path_rev->revnum,
-                                       iter_pool));
-          SVN_ERR(authz_read_func(&readable, tmp_root, path_rev->path,
-                                  authz_read_baton, iter_pool));
+          SVN_ERR(svn_fs_revision_root(&tmp_root, repos->fs, tmp_revnum,
+                                       iterpool));
+          SVN_ERR(authz_read_func(&readable, tmp_root, tmp_path,
+                                  authz_read_baton, iterpool));
           if (! readable)
             break;
         }
 
-      path_rev->path = apr_pstrdup(pool, path_rev->path);
+      /* We didn't break, so we must really want this path-rev. */
+      path_rev = apr_palloc(result_pool, sizeof(*path_rev));
+      path_rev->path = apr_pstrdup(result_pool, tmp_path);
+      path_rev->revnum = tmp_revnum;
       path_rev->merged = mark_as_merged;
       APR_ARRAY_PUSH(path_revisions, struct path_revision *) = path_rev;
 
       if (include_merged_revisions)
         SVN_ERR(get_merged_mergeinfo(&path_rev->merged_mergeinfo, repos,
-                                     path_rev, pool));
+                                     path_rev, result_pool));
       else
         path_rev->merged_mergeinfo = NULL;
 
@@ -1170,7 +1187,7 @@ find_interesting_revisions(apr_array_header_t *path_revisions,
          occurrences of it.  We only care about this if including merged
          revisions, 'cause that's the only time we can have duplicates. */
       apr_hash_set(duplicate_path_revs,
-                   apr_psprintf(pool, "%s:%ld", path_rev->path,
+                   apr_psprintf(result_pool, "%s:%ld", path_rev->path,
                                 path_rev->revnum),
                    APR_HASH_KEY_STRING, (void *)0xdeadbeef);
 
@@ -1178,12 +1195,13 @@ find_interesting_revisions(apr_array_header_t *path_revisions,
         break;
 
       /* Swap pools. */
-      tmp_pool = iter_pool;
-      iter_pool = last_pool;
+      tmp_pool = iterpool;
+      iterpool = last_pool;
       last_pool = tmp_pool;
     }
 
-  svn_pool_destroy(iter_pool);
+  svn_pool_destroy(iterpool);
+  svn_pool_destroy(last_pool);
 
   return SVN_NO_ERROR;
 }
@@ -1211,12 +1229,12 @@ find_merged_revisions(apr_array_header_t **merged_path_revisions_out,
                       apr_pool_t *pool)
 {
   apr_array_header_t *old, *new;
-  apr_pool_t *iter_pool, *last_pool;
+  apr_pool_t *iterpool, *last_pool;
   apr_array_header_t *merged_path_revisions = apr_array_make(pool, 0,
                                                 sizeof(struct path_revision *));
 
   old = mainline_path_revisions;
-  iter_pool = svn_pool_create(pool);
+  iterpool = svn_pool_create(pool);
   last_pool = svn_pool_create(pool);
 
   do
@@ -1224,27 +1242,34 @@ find_merged_revisions(apr_array_header_t **merged_path_revisions_out,
       int i;
       apr_pool_t *temp_pool;
 
-      svn_pool_clear(iter_pool);
-      new = apr_array_make(iter_pool, 0, sizeof(struct path_revision *));
+      svn_pool_clear(iterpool);
+      new = apr_array_make(iterpool, 0, sizeof(struct path_revision *));
 
       /* Iterate over OLD, checking for non-empty mergeinfo.  If found, gather
          path_revisions for any merged revisions, and store those in NEW. */
       for (i = 0; i < old->nelts; i++)
         {
+          apr_pool_t *iterpool2;
           apr_hash_index_t *hi;
           struct path_revision *old_pr = APR_ARRAY_IDX(old, i,
                                                        struct path_revision *);
           if (!old_pr->merged_mergeinfo)
             continue;
 
+          iterpool2 = svn_pool_create(iterpool);
+
           /* Determine and trace the merge sources. */
-          for (hi = apr_hash_first(iter_pool, old_pr->merged_mergeinfo); hi;
+          for (hi = apr_hash_first(iterpool, old_pr->merged_mergeinfo); hi;
                hi = apr_hash_next(hi))
             {
+              apr_pool_t *iterpool3;
               apr_array_header_t *rangelist;
               const char *path;
               int j;
 
+              svn_pool_clear(iterpool2);
+              iterpool3 = svn_pool_create(iterpool2);
+
               apr_hash_this(hi, (void *) &path, NULL, (void *) &rangelist);
 
               for (j = 0; j < rangelist->nelts; j++)
@@ -1254,9 +1279,10 @@ find_merged_revisions(apr_array_header_t **merged_path_revisions_out,
                   svn_node_kind_t kind;
                   svn_fs_root_t *root;
 
+                  svn_pool_clear(iterpool3);
                   SVN_ERR(svn_fs_revision_root(&root, repos->fs, range->end,
-                                               iter_pool));
-                  SVN_ERR(svn_fs_check_path(&kind, root, path, iter_pool));
+                                               iterpool3));
+                  SVN_ERR(svn_fs_check_path(&kind, root, path, iterpool3));
                   if (kind != svn_node_file)
                     continue;
 
@@ -1266,20 +1292,23 @@ find_merged_revisions(apr_array_header_t **merged_path_revisions_out,
                                                      TRUE, TRUE,
                                                      duplicate_path_revs,
                                                      authz_read_func,
-                                                     authz_read_baton, pool));
+                                                     authz_read_baton, pool,
+                                                     iterpool3));
                 }
+              svn_pool_destroy(iterpool3);
             }
+          svn_pool_destroy(iterpool2);
         }
 
       /* Append the newly found path revisions with the old ones. */
-      merged_path_revisions = apr_array_append(iter_pool, merged_path_revisions,
+      merged_path_revisions = apr_array_append(iterpool, merged_path_revisions,
                                                new);
 
       /* Swap data structures */
       old = new;
       temp_pool = last_pool;
-      last_pool = iter_pool;
-      iter_pool = temp_pool;
+      last_pool = iterpool;
+      iterpool = temp_pool;
     }
   while (new->nelts > 0);
 
@@ -1290,7 +1319,7 @@ find_merged_revisions(apr_array_header_t **merged_path_revisions_out,
   /* Copy to the output array. */
   *merged_path_revisions_out = apr_array_copy(pool, merged_path_revisions);
 
-  svn_pool_destroy(iter_pool);
+  svn_pool_destroy(iterpool);
   svn_pool_destroy(last_pool);
 
   return SVN_NO_ERROR;
@@ -1397,8 +1426,8 @@ send_path_revision(struct path_revision *path_rev,
  *     merged revisions, including them in the MERGED_PATH_REVISIONS, and using
  *     DUPLICATE_PATH_REVS to avoid tracing the same paths of history multiple
  *     times.
- *  3) Send both MAINLINE_PATH_REVISIONS and MERGED_PATH_REVISIONS from 
- *     youngest to oldest, interleaving as appropriate.  This is implemented
+ *  3) Send both MAINLINE_PATH_REVISIONS and MERGED_PATH_REVISIONS from
+ *     oldest to youngest, interleaving as appropriate.  This is implemented
  *     similar to an insertion sort, but instead of inserting into another
  *     array, we just call the appropriate handler.
  */
@@ -1426,7 +1455,8 @@ svn_repos_get_file_revs2(svn_repos_t *repos,
   SVN_ERR(find_interesting_revisions(mainline_path_revisions, repos, path,
                                      start, end, include_merged_revisions,
                                      FALSE, duplicate_path_revs,
-                                     authz_read_func, authz_read_baton, pool));
+                                     authz_read_func, authz_read_baton, pool,
+                                     pool));
 
   /* If we are including merged revisions, go get those, too. */
   if (include_merged_revisions)
@@ -1439,7 +1469,7 @@ svn_repos_get_file_revs2(svn_repos_t *repos,
                                            sizeof(struct path_revision *));
 
   /* We must have at least one revision to get. */
-  assert(mainline_path_revisions->nelts > 0);
+  SVN_ERR_ASSERT(mainline_path_revisions->nelts > 0);
 
   /* We switch betwwen two pools while looping, since we need information from
      the last iteration to be available. */
@@ -1465,7 +1495,7 @@ svn_repos_get_file_revs2(svn_repos_t *repos,
       struct path_revision *merged_pr = APR_ARRAY_IDX(merged_path_revisions,
                                                       merged_pos,
                                                       struct path_revision *);
-    
+
       if (main_pr->revnum <= merged_pr->revnum)
         {
           SVN_ERR(send_path_revision(main_pr, repos, &sb, handler,
@@ -1504,25 +1534,3 @@ svn_repos_get_file_revs2(svn_repos_t *repos,
 
   return SVN_NO_ERROR;
 }
-
-svn_error_t *
-svn_repos_get_file_revs(svn_repos_t *repos,
-                        const char *path,
-                        svn_revnum_t start,
-                        svn_revnum_t end,
-                        svn_repos_authz_func_t authz_read_func,
-                        void *authz_read_baton,
-                        svn_repos_file_rev_handler_t handler,
-                        void *handler_baton,
-                        apr_pool_t *pool)
-{
-  svn_file_rev_handler_t handler2;
-  void *handler2_baton;
-
-  svn_compat_wrap_file_rev_handler(&handler2, &handler2_baton, handler,
-                                   handler_baton, pool);
-
-  return svn_repos_get_file_revs2(repos, path, start, end, FALSE,
-                                  authz_read_func, authz_read_baton,
-                                  handler2, handler2_baton, pool);
-}
