@@ -39,7 +39,7 @@ extern "C" {
    and creating a lock. The cleanup code uses this function, but stealing
    locks is not a good idea because the code cannot determine whether a
    lock is still in use. Try not to write any more code that requires this
-   feature. 
+   feature.
 
    PATH is the directory to lock, and the lock is returned in
    *ADM_ACCESS.  ASSOCIATED can be another lock in which case the locks
@@ -52,9 +52,10 @@ svn_error_t *svn_wc__adm_steal_write_lock(svn_wc_adm_access_t **adm_access,
 
 /* Set *CLEANUP to TRUE if the directory ADM_ACCESS requires cleanup
    processing, set *CLEANUP to FALSE otherwise. */
-svn_error_t *svn_wc__adm_is_cleanup_required(svn_boolean_t *cleanup,
-                                             svn_wc_adm_access_t *adm_access,
-                                             apr_pool_t *pool);
+svn_error_t *
+svn_wc__adm_is_cleanup_required(svn_boolean_t *cleanup,
+                                const svn_wc_adm_access_t *adm_access,
+                                apr_pool_t *pool);
 
 /* Store ENTRIES in the cache in ADM_ACCESS appropriate for SHOW_DELETED.
    ENTRIES may be NULL. */
@@ -74,7 +75,7 @@ void svn_wc__adm_access_set_wcprops(svn_wc_adm_access_t *adm_access,
                                     apr_hash_t *wcprops);
 
 /* Retrieve the wcprops cached in ADM_ACCESS, if any. */
-apr_hash_t *svn_wc__adm_access_wcprops(svn_wc_adm_access_t *adm_access);
+apr_hash_t *svn_wc__adm_access_wcprops(const svn_wc_adm_access_t *adm_access);
 
 
 
@@ -91,20 +92,20 @@ svn_error_t *svn_wc__adm_pre_open(svn_wc_adm_access_t **adm_access,
    the parent of PATH and when that access baton was opened it must have
    attempted to open PATH, i.e. it must have been opened with the TREE_LOCK
    parameter set TRUE. */
-svn_boolean_t svn_wc__adm_missing(svn_wc_adm_access_t *adm_access,
+svn_boolean_t svn_wc__adm_missing(const svn_wc_adm_access_t *adm_access,
                                   const char *path);
 
 /* Sets *ADM_ACCESS to an access baton for PATH from the set ASSOCIATED.
    This function is similar to svn_wc_adm_retrieve except that if the baton
    for PATH is not found, this function sets *ADM_ACCESS to NULL and does
    not return an error. */
-svn_error_t * svn_wc__adm_retrieve_internal(svn_wc_adm_access_t **adm_access,
-                                            svn_wc_adm_access_t *associated,
-                                            const char *path,
-                                            apr_pool_t *pool);
+svn_error_t *svn_wc__adm_retrieve_internal(svn_wc_adm_access_t **adm_access,
+                                           svn_wc_adm_access_t *associated,
+                                           const char *path,
+                                           apr_pool_t *pool);
 
 /* Return the working copy format version number for ADM_ACCESS. */
-int svn_wc__adm_wc_format(svn_wc_adm_access_t *adm_access);
+int svn_wc__adm_wc_format(const svn_wc_adm_access_t *adm_access);
 
 /* Set the WC FORMAT of this access baton. */
 void svn_wc__adm_set_wc_format(svn_wc_adm_access_t *adm_access,
@@ -117,7 +118,16 @@ void svn_wc__adm_set_wc_format(svn_wc_adm_access_t *adm_access,
  * when the library expects a write lock, and where it is an error for the
  * lock not to be present.  Applications are not expected to call it.
  */
-svn_error_t *svn_wc__adm_write_check(svn_wc_adm_access_t *adm_access);
+svn_error_t *svn_wc__adm_write_check(const svn_wc_adm_access_t *adm_access,
+                                     apr_pool_t *scratch_pool);
+
+/* Ensure ADM_ACCESS has a lock and for an entire WC tree (all the way
+   to its leaf nodes).  While locking a tree up front using
+   LEVELS_TO_LOCK of -1 is a more appropriate operation, this function
+   can be used to extend the depth of a lock via a tree-crawl after a
+   lock is taken out.  Use POOL for temporary allocations. */
+svn_error_t *svn_wc__adm_extend_lock_to_tree(svn_wc_adm_access_t *adm_access,
+                                             apr_pool_t *pool);
 
 #ifdef __cplusplus
 }
