@@ -125,7 +125,7 @@ typedef struct {
      constants anyway (and sufficiently well-informed internal code
      may therefore compare against those constants' addresses).  If
      'is_svn_client' is false, then 'capabilities' should be empty. */
-  apr_hash_t *capabilities;
+  apr_hash_t *client_capabilities;
 
   /* The path to the activities db */
   const char *activities_db;
@@ -250,6 +250,10 @@ struct dav_resource_private {
   /* was this resource auto-checked-out? */
   svn_boolean_t auto_checked_out;
 
+  /* was this resource fetched using our public peg-/working-rev CGI
+     interface (ie: /path/to/item?p=PEGREV]? */
+  svn_boolean_t pegged;
+
   /* Pool to allocate temporary data from */
   apr_pool_t *pool;
 };
@@ -500,7 +504,7 @@ dav_error *
 dav_svn__merge_response(ap_filter_t *output,
                         const dav_svn_repos *repos,
                         svn_revnum_t new_rev,
-                        char *post_commit_err,
+                        const char *post_commit_err,
                         apr_xml_elem *prop_elem,
                         svn_boolean_t disable_merge_response,
                         apr_pool_t *pool);
@@ -520,6 +524,7 @@ static const dav_report_elem dav_svn__reports_list[] = {
   { SVN_XML_NAMESPACE, "file-revs-report" },
   { SVN_XML_NAMESPACE, "get-locks-report" },
   { SVN_XML_NAMESPACE, "replay-report" },
+  { SVN_XML_NAMESPACE, "get-deleted-rev-report" },
   { SVN_XML_NAMESPACE, SVN_DAV__MERGEINFO_REPORT },
   { NULL, NULL },
 };
@@ -563,6 +568,10 @@ dav_svn__get_locks_report(const dav_resource *resource,
                           const apr_xml_doc *doc,
                           ap_filter_t *output);
 
+dav_error *
+dav_svn__get_deleted_rev_report(const dav_resource *resource,
+                                const apr_xml_doc *doc,
+                                ap_filter_t *output);
 
 /*** authz.c ***/
 
@@ -578,17 +587,32 @@ typedef struct
 } dav_svn__authz_read_baton;
 
 
-/* Convert incoming RESOURCE and revision REV into a version-resource URI and
-   perform a GET subrequest on it.  This will invoke any authz modules loaded
-   into apache. Return TRUE if the subrequest succeeds, FALSE otherwise.
-
-   If REV is SVN_INVALID_REVNUM, then we look at HEAD.
-   Use POOL for any temporary allocation.
+/* Return TRUE iff the current user (as determined by Apache's
+   authentication system) has permission to read PATH in REPOS at REV
+   (where an invalid REV means "HEAD").  This will invoke any authz
+   modules loaded into Apache unless this Subversion location has been
+   configured to bypass those in favor of a direct lookup in the
+   Subversion authz subsystem.  Use POOL for any temporary allocation.
 */
 svn_boolean_t
-dav_svn__allow_read(const dav_resource *resource,
-                   svn_revnum_t rev,
-                   apr_pool_t *pool);
+dav_svn__allow_read(request_rec *r,
+                    const dav_svn_repos *repos,
+                    const char *path,
+                    svn_revnum_t rev,
+                    apr_pool_t *pool);
+
+/* Return TRUE iff the current user (as determined by Apache's
+   authentication system) has permission to read RESOURCE in REV
+   (where an invalid REV means "HEAD").  This will invoke any authz
+   modules loaded into Apache unless this Subversion location has been
+   configured to bypass those in favor of a direct lookup in the
+   Subversion authz subsystem.  Use POOL for any temporary allocation.
+*/
+svn_boolean_t
+dav_svn__allow_read_resource(const dav_resource *resource,
+                             svn_revnum_t rev,
+                             apr_pool_t *pool);
+
 
 /* If authz is enabled in the specified BATON, return a read authorization
    function. Otherwise, return NULL. */
@@ -754,6 +778,43 @@ dav_svn__make_base64_output_stream(apr_bucket_brigade *bb,
 void
 dav_svn__operational_log(struct dav_resource_private *info, const char *line);
 
+/* Flush BB if it's okay and useful to do so, but treat PREFERRED_ERR
+ * as a more important error to return (if it is non-NULL).
+ *
+ * This is intended to be used at the end of response processing,
+ * probably called as a direct return generator, like so:
+ *
+ *   return dav_svn__final_flush_or_error(r, bb, output, derr, resource->pool);
+ *
+ * SOME BACKGROUND INFO:
+ *
+ * We don't flush the brigade unless there's something in it to
+ * flush; that way, we have the opportunity to package a dav_error up
+ * for transmission back to the client.
+ *
+ * To understand this, see mod_dav.c:dav_method_report(): as long as
+ * it doesn't think we've sent anything to the client, it'll send
+ * the real error, which is what we'd prefer.  This situation is
+ * described in httpd-2.2.6/modules/dav/main/mod_dav.c, line 4066,
+ * in the comment in dav_method_report() that says:
+ *
+ *    If an error occurred during the report delivery, there's
+ *    basically nothing we can do but abort the connection and
+ *    log an error.  This is one of the limitations of HTTP; it
+ *    needs to "know" the entire status of the response before
+ *    generating it, which is just impossible in these streamy
+ *    response situations.
+ *
+ * In other words, flushing the brigade causes r->sent_bodyct (see
+ * dav_method_report()) to become non-zero, even if we hadn't tried to
+ * send any data to the brigade yet.  So we don't flush unless data
+ * was actually sent.
+ */
+dav_error *
+dav_svn__final_flush_or_error(request_rec *r, apr_bucket_brigade *bb,
+                              ap_filter_t *output, dav_error *preferred_err,
+                              apr_pool_t *pool);
+
 /*** mirror.c ***/
 
 /* Perform the fixup hook for the R request.  */
