@@ -33,6 +33,11 @@
 #include "svn_config.h"
 #include "cl.h"
 
+#include "svn_private_config.h"
+
+/* We shouldn't be including a private header here, but it is
+ * necessary for fixing issue #3416 */
+#include "private/svn_opt_private.h"
 
 
 /* This implements the `svn_opt_subcommand_t' interface. */
@@ -53,8 +58,8 @@ svn_cl__commit(apr_getopt_t *os,
   int i;
 
   SVN_ERR(svn_cl__args_to_target_array_print_reserved(&targets, os,
-                                                      opt_state->targets, 
-                                                      pool));
+                                                      opt_state->targets,
+                                                      ctx, pool));
 
   /* Check that no targets are URLs */
   for (i = 0; i < targets->nelts; i++)
@@ -69,6 +74,8 @@ svn_cl__commit(apr_getopt_t *os,
   /* Add "." if user passed 0 arguments. */
   svn_opt_push_implicit_dot_target(targets, pool);
 
+  SVN_ERR(svn_opt__eat_peg_revisions(&targets, targets, pool));
+
   /* Condense the targets (like commit does)... */
   SVN_ERR(svn_path_condense_targets(&base_dir,
                                     &condensed_targets,
@@ -93,6 +100,21 @@ svn_cl__commit(apr_getopt_t *os,
   if (opt_state->depth == svn_depth_unknown)
     opt_state->depth = svn_depth_infinity;
 
+  /* Copies are done server-side, and cheaply, which means they're
+   * effectively always done with infinite depth.
+   * This is a potential cause of confusion for users trying to commit
+   * copied subtrees in part by restricting the commit's depth.
+   * See issue #3699. */
+  if (opt_state->depth < svn_depth_infinity)
+    SVN_ERR(svn_cmdline_printf(pool,
+                               _("svn: warning: The depth of this commit "
+                                 "is '%s', but copied directories will "
+                                 "regardless be committed with depth '%s'. "
+                                 "You must remove unwanted children of those "
+                                 "directories in a separate commit.\n"),
+                               svn_depth_to_word(opt_state->depth),
+                               svn_depth_to_word(svn_depth_infinity)));
+
   cfg = apr_hash_get(ctx->config, SVN_CONFIG_CATEGORY_CONFIG,
                      APR_HASH_KEY_STRING);
   if (cfg)
@@ -132,7 +154,7 @@ svn_cl__commit(apr_getopt_t *os,
           err = root_err;
         }
     }
-  SVN_ERR(svn_cl__cleanup_log_msg(ctx->log_msg_baton3, err));
+  SVN_ERR(svn_cl__cleanup_log_msg(ctx->log_msg_baton3, err, pool));
   if (! err && ! opt_state->quiet)
     SVN_ERR(svn_cl__print_commit_info(commit_info, pool));
 
