@@ -6,7 +6,7 @@
 #  See http://subversion.tigris.org for more information.
 #
 # ====================================================================
-# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2009 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -48,7 +48,7 @@ def status_unversioned_file_in_current_dir(sbox):
 
   svntest.main.file_append('foo', 'a new file')
 
-  svntest.actions.run_and_verify_svn(None, [ "?      foo\n" ], [],
+  svntest.actions.run_and_verify_svn(None, [ "?       foo\n" ], [],
                                      'stat', 'foo')
 
 #----------------------------------------------------------------------
@@ -94,18 +94,11 @@ def status_update_with_nested_adds(sbox):
   # Now we go to the backup working copy, still at revision 1.
   # We will run 'svn st -u', and make sure that newdir/newfile is reported
   # as a nonexistent (but pending) path.
-
-  # Create expected status tree; all local revisions should be at 1,
-  # but newdir and newfile should be present with 'blank' attributes.
   expected_status = svntest.actions.get_virginal_state(wc_backup, 1)
-
-  # Verify status.  Notice that we're running status *without* the
-  # --quiet flag, so the unversioned items will appear.
-  # Unfortunately, the regexp that we currently use to parse status
-  # output is unable to parse a line that has no working revision!  If
-  # an error happens, we'll catch it here.  So that's a good enough
-  # regression test for now.  Someday, though, it would be nice to
-  # positively match the mostly-empty lines.
+  expected_status.add({
+    'newdir'         : Item(status='  '),
+    'newdir/newfile' : Item(status='  '),
+    })
   svntest.actions.run_and_verify_unquiet_status(wc_backup,
                                                 expected_status)
 
@@ -119,8 +112,8 @@ def status_shows_all_in_current_dir(sbox):
   wc_dir = sbox.wc_dir
 
   os.chdir(wc_dir)
-  output, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                   'stat', '-vN')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'stat', '-vN')
 
   if (len(output) != len(os.listdir("."))):
     raise svntest.Failure
@@ -138,14 +131,16 @@ def status_missing_file(sbox):
 
   os.remove('iota')
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status')
   for line in output:
     if not re.match("! +iota", line):
       raise svntest.Failure
 
   # This invocation is for issue #2127.
-  output, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                   'status', '-u', 'iota')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status', '-u',
+                                                              'iota')
   found_it = 0
   for line in output:
     if re.match("! +1 +iota", line):
@@ -169,7 +164,8 @@ def status_type_change(sbox):
   os.rename('A', 'iota')
   os.rename('was_iota', 'A')
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status')
   if len(output) != 2:
     raise svntest.Failure
   for line in output:
@@ -181,7 +177,8 @@ def status_type_change(sbox):
   os.remove('A')
   os.mkdir('A')
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status')
   if len(output) != 2:
     raise svntest.Failure
   for line in output:
@@ -193,7 +190,8 @@ def status_type_change(sbox):
   svntest.main.safe_rmtree('iota')
   os.mkdir('iota')
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status')
   if len(output) != 2:
     raise svntest.Failure
   for line in output:
@@ -216,7 +214,8 @@ def status_type_change_to_symlink(sbox):
   svntest.main.safe_rmtree('A/D')
   os.symlink('bar', 'A/D')
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status')
   if len(output) != 2:
     raise svntest.Failure
   for line in output:
@@ -229,7 +228,8 @@ def status_type_change_to_symlink(sbox):
   os.symlink('A/mu', 'iota')
   os.symlink('C', 'A/D')
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [], 'status')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status')
   if len(output) != 2:
     raise svntest.Failure
   for line in output:
@@ -254,16 +254,12 @@ def status_with_new_files_pending(sbox):
   svntest.main.run_svn(None,
                        'up', '-r', '1')
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                   'status', '-u')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                              'status', '-u')
 
-  # The bug fixed in revision 3686 was a seg fault.  We don't have a
-  # reliable way to detect a seg fault here, since we haven't dealt
-  # with the popen2{Popen3,Popen4} mess in Python yet (the latter two
-  # are classes within the first, which is a module, and the Popen3
-  # class is not the same as os.popen3().  Got that?)  See the Python
-  # docs for details; in the meantime, no output means there was a
-  # problem.
+  # The bug fixed in revision 3686 was a segmentation fault.
+  # TODO: Check exit code.
+  # In the meantime, no output means there was a problem.
   for line in output:
     if line.find('newfile') != -1:
       break
@@ -290,9 +286,9 @@ def status_for_unignored_file(sbox):
 
   # status on the directory with --no-ignore
   expected = svntest.verify.UnorderedOutput(
-        ['I      newdir\n',
-         'I      newfile\n',
-         ' M     .\n'])
+        ['I       newdir\n',
+         'I       newfile\n',
+         ' M      .\n'])
   svntest.actions.run_and_verify_svn(None,
                                      expected,
                                      [],
@@ -300,8 +296,8 @@ def status_for_unignored_file(sbox):
 
   # status specifying the file explicitly on the command line
   expected = svntest.verify.UnorderedOutput(
-        ['I      newdir\n',
-         'I      newfile\n'])
+        ['I       newdir\n',
+         'I       newfile\n'])
   svntest.actions.run_and_verify_svn(None,
                                      expected,
                                      [],
@@ -318,9 +314,8 @@ def status_for_nonexistent_file(sbox):
 
   os.chdir(wc_dir)
 
-  output, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                   'status',
-                                                   'nonexistent-file')
+  exit_code, output, err = svntest.actions.run_and_verify_svn(
+    None, None, [], 'status', 'nonexistent-file')
 
   # there should *not* be a status line printed for the nonexistent file
   for line in output:
@@ -353,9 +348,9 @@ def status_nonrecursive_update_different_cwd(sbox):
   os.chdir(wc_dir)
 
   expected_output = [
-    '       *                                %s\n' % os.path.join("C", "J"),
-    'A               0       ?   ?           %s\n' % os.path.join("C", "K"),
-    '       *        1        1 jrandom      C\n',
+    '        *                                %s\n' % os.path.join("C", "J"),
+    'A                0       ?   ?           %s\n' % os.path.join("C", "K"),
+    '        *        1        1 jrandom      C\n',
     'Status against revision:      2\n' ]
 
   os.chdir('A')
@@ -365,9 +360,9 @@ def status_nonrecursive_update_different_cwd(sbox):
                                      'status', '-v', '-N', '-u', 'C')
 
   expected_output = [
-    '       *                                J\n',
-    'A               0       ?   ?           K\n',
-    '       *        1        1 jrandom      .\n',
+    '        *                                J\n',
+    'A                0       ?   ?           K\n',
+    '        *        1        1 jrandom      .\n',
     'Status against revision:      2\n']
 
   os.chdir('C')
@@ -435,8 +430,9 @@ def status_file_needs_update(sbox):
   # the -v flag, which we don't want, as this bug never appeared when
   # -v was passed.  So we run status by hand:
   os.chdir(was_cwd)
-  out, err = svntest.actions.run_and_verify_svn(None, None, [],
-                                                'status', '-u', other_wc)
+  exit_code, out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                           'status', '-u',
+                                                           other_wc)
 
   for line in out:
     if re.match("\\s+\\*.*crontab\\.root$", line):
@@ -497,7 +493,7 @@ def status_uninvited_parent_directory(sbox):
   # We don't want a full status tree here, just one line (or two, if
   # the bug is present).  So run status by hand:
   os.chdir(was_cwd)
-  out, err = svntest.actions.run_and_verify_svn(
+  exit_code, out, err = svntest.actions.run_and_verify_svn(
     None, None, [],
     'status', '-u', os.path.join(other_wc, 'newfile'))
 
@@ -552,21 +548,23 @@ def status_on_forward_deletion(sbox):
 
 def get_last_changed_date(path):
   "get the Last Changed Date for path using svn info"
-  out, err = svntest.actions.run_and_verify_svn(None, None, [], 'info', path)
+  exit_code, out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                           'info', path)
   for line in out:
     if re.match("^Last Changed Date", line):
       return line
-  print "Didn't find Last Changed Date for " + path
+  print("Didn't find Last Changed Date for " + path)
   raise svntest.Failure
 
 # Helper for timestamp_behaviour test
 def get_text_timestamp(path):
   "get the text-time for path using svn info"
-  out, err = svntest.actions.run_and_verify_svn(None, None, [], 'info', path)
+  exit_code, out, err = svntest.actions.run_and_verify_svn(None, None, [],
+                                                           'info', path)
   for line in out:
     if re.match("^Text Last Updated", line):
       return line
-  print "Didn't find text-time for " + path
+  print("Didn't find text-time for " + path)
   raise svntest.Failure
 
 # Helper for timestamp_behaviour test
@@ -672,7 +670,7 @@ def status_on_unversioned_dotdot(sbox):
 
   os.chdir(new_subdir)
 
-  out, err = svntest.main.run_svn(1, 'st', '..')
+  exit_code, out, err = svntest.main.run_svn(1, 'st', '..')
   for line in err:
     if line.find('svn: warning: \'..\' is not a working copy') != -1:
       break
@@ -772,9 +770,9 @@ def status_in_xml(sbox):
   svntest.main.file_append(file_path, "test status --xml\n")
 
   # Retrieve last changed date from svn log
-  output, error = svntest.actions.run_and_verify_svn(None, None, [],
-                                                     'log', file_path,
-                                                     '--xml', '-rHEAD')
+  exit_code, output, error = svntest.actions.run_and_verify_svn(
+    None, None, [], 'log', file_path, '--xml', '-rHEAD')
+
   info_msg = "<date>"
   for line in output:
     if line.find(info_msg) >= 0:
@@ -806,13 +804,14 @@ def status_in_xml(sbox):
               "</status>\n",
              ]
 
-  output, error = svntest.actions.run_and_verify_svn(None, None, [],
-                                                     'status', file_path,
-                                                     '--xml', '-u')
+  exit_code, output, error = svntest.actions.run_and_verify_svn(None, None, [],
+                                                                'status',
+                                                                file_path,
+                                                                '--xml', '-u')
 
   for i in range(0, len(output)):
     if output[i] != template[i]:
-      print "ERROR: expected:", template[i], "actual:", output[i]
+      print("ERROR: expected: %s actual: %s" % (template[i], output[i]))
       raise svntest.Failure
 
 #----------------------------------------------------------------------
@@ -832,8 +831,8 @@ def status_ignored_dir(sbox):
 
   # run_and_verify_status doesn't handle this weird kind of entry.
   svntest.actions.run_and_verify_svn(None,
-                                     ['I      *            ' + new_dir + "\n",
-                                      '       *        1   ' + wc_dir + "\n",
+                                     ['I       *            ' + new_dir + "\n",
+                                      '        *        1   ' + wc_dir + "\n",
                                       'Status against revision:      2\n'], [],
                                      "status", "-u", wc_dir)
 
@@ -845,7 +844,8 @@ def status_unversioned_dir(sbox):
   dir = sbox.repo_dir
   expected_err = ["svn: warning: '" + dir + "' is not a working copy\n",
                   "svn: warning: '" + dir + "' is not a working copy\n"]
-  svntest.actions.run_and_verify_svn(None, [], expected_err, "status", dir, dir)
+  svntest.actions.run_and_verify_svn2(None, [], expected_err, 0,
+                                      "status", dir, dir)
 
 #----------------------------------------------------------------------
 
@@ -858,15 +858,15 @@ def status_missing_dir(sbox):
   # ok, blow away the A/D/G directory
   svntest.main.safe_rmtree(a_d_g)
 
-  expected = svntest.verify.UnorderedOutput(["!      " + a_d_g + "\n"])
+  expected = svntest.verify.UnorderedOutput(["!       " + a_d_g + "\n"])
   svntest.actions.run_and_verify_svn(None, expected, [], "status", wc_dir)
 
   expected = svntest.verify.UnorderedOutput(
-         ["       *            " + os.path.join(a_d_g, "pi") + "\n",
-          "       *            " + os.path.join(a_d_g, "rho") + "\n",
-          "       *            " + os.path.join(a_d_g, "tau") + "\n",
-          "!      *       ?    " + a_d_g + "\n",
-          "       *        1   " + os.path.join(wc_dir, "A", "D") + "\n",
+         ["        *            " + os.path.join(a_d_g, "pi") + "\n",
+          "        *            " + os.path.join(a_d_g, "rho") + "\n",
+          "        *            " + os.path.join(a_d_g, "tau") + "\n",
+          "!       *       ?    " + a_d_g + "\n",
+          "        *        1   " + os.path.join(wc_dir, "A", "D") + "\n",
           "Status against revision:      1\n" ])
 
   # now run status -u, we should be able to do this without crashing
@@ -923,13 +923,13 @@ def status_add_plus_conflict(sbox):
                                      branch_url, '-r', '4:5', trunk_dir)
 
   lines = [
-    "?      " + os.path.join(wc_dir, "trunk", "file.merge-left.r4") + "\n",
-    "?      " + os.path.join(wc_dir, "trunk", "file.merge-right.r5") + "\n",
-    "?      " + os.path.join(wc_dir, "trunk", "file.working") + "\n",
-    "C  +   " + os.path.join(wc_dir, "trunk", "file") + "\n",
+    "?       " + os.path.join(wc_dir, "trunk", "file.merge-left.r4") + "\n",
+    "?       " + os.path.join(wc_dir, "trunk", "file.merge-right.r5") + "\n",
+    "?       " + os.path.join(wc_dir, "trunk", "file.working") + "\n",
+    "C  +    " + os.path.join(wc_dir, "trunk", "file") + "\n",
   ]
   if svntest.main.server_has_mergeinfo():
-    lines.append(" M     " + os.path.join(wc_dir, "trunk") + "\n")
+    lines.append(" M      " + os.path.join(wc_dir, "trunk") + "\n")
 
   expected_output = svntest.verify.UnorderedOutput(lines)
 
@@ -1015,8 +1015,8 @@ def status_update_with_incoming_props(sbox):
   # Can't use run_and_verify_status here because the out-of-date
   # information in the status output isn't copied in the status tree.
   expected = svntest.verify.UnorderedOutput(
-         ["       *        1   " + A_path + "\n",
-          "       *        1   " + wc_dir + "\n",
+         ["        *        1   " + A_path + "\n",
+          "        *        1   " + wc_dir + "\n",
           "Status against revision:      2\n" ])
 
   svntest.actions.run_and_verify_svn(None,
@@ -1026,10 +1026,10 @@ def status_update_with_incoming_props(sbox):
                                      wc_dir)
 
   expected = svntest.verify.UnorderedOutput(
-         ["                1        1 jrandom      " +
+         ["                 1        1 jrandom      " +
           os.path.join(wc_dir, "iota") + "\n",
-          "       *        1        1 jrandom      " + A_path + "\n",
-          "       *        1        1 jrandom      " + wc_dir + "\n",
+          "        *        1        1 jrandom      " + A_path + "\n",
+          "        *        1        1 jrandom      " + wc_dir + "\n",
           "Status against revision:      2\n" ])
 
   svntest.actions.run_and_verify_svn(None, expected, [],
@@ -1037,9 +1037,9 @@ def status_update_with_incoming_props(sbox):
                                      wc_dir)
 
   # Retrieve last changed date from svn log
-  output, error = svntest.actions.run_and_verify_svn(None, None, [],
-                                                     'log', wc_dir,
-                                                     '--xml', '-r1')
+  exit_code, output, error = svntest.actions.run_and_verify_svn(None, None, [],
+                                                                'log', wc_dir,
+                                                                '--xml', '-r1')
 
   info_msg = "<date>"
   for line in output:
@@ -1092,9 +1092,10 @@ def status_update_with_incoming_props(sbox):
           "</target>\n",
           "</status>\n",]
 
-  output, error = svntest.actions.run_and_verify_svn(None, xout, [],
-                                                     'status', wc_dir,
-                                                     '--xml', '-uN')
+  exit_code, output, error = svntest.actions.run_and_verify_svn(None, xout, [],
+                                                                'status',
+                                                                wc_dir,
+                                                                '--xml', '-uN')
 
 # more incoming prop updates.
 def status_update_verbose_with_incoming_props(sbox):
@@ -1151,27 +1152,27 @@ def status_update_verbose_with_incoming_props(sbox):
   # information in the status output isn't copied in the status tree.
   common = "        1        1 jrandom      "
   expected = svntest.verify.UnorderedOutput(
-         ["        " + common + os.path.join(E_path, 'alpha') + "\n",
-          "        " + common + os.path.join(E_path, 'beta') + "\n",
-          "       *" + common + os.path.join(E_path) + "\n",
-          "        " + common + os.path.join(B_path, 'lambda') + "\n",
-          "        " + common + os.path.join(B_path, 'F') + "\n",
-          "        " + common + B_path + "\n",
-          "        " + common + os.path.join(G_path, 'pi') + "\n",
-          "        " + common + os.path.join(G_path, 'rho') + "\n",
-          "        " + common + os.path.join(G_path, 'tau') + "\n",
-          "        " + common + G_path + "\n",
-          "        " + common + os.path.join(H_path, 'chi') + "\n",
-          "        " + common + os.path.join(H_path, 'omega') + "\n",
-          "        " + common + os.path.join(H_path, 'psi') + "\n",
-          "        " + common + H_path + "\n",
-          "        " + common + os.path.join(D_path, 'gamma') + "\n",
-          "       *" + common + D_path + "\n",
-          "        " + common + os.path.join(A_path, 'mu') + "\n",
-          "        " + common + os.path.join(A_path, 'C') + "\n",
-          "        " + common + A_path + "\n",
-          "        " + common + os.path.join(wc_dir, 'iota') + "\n",
-          "       *" + common + wc_dir  + "\n",
+         ["         " + common + os.path.join(E_path, 'alpha') + "\n",
+          "         " + common + os.path.join(E_path, 'beta') + "\n",
+          "        *" + common + os.path.join(E_path) + "\n",
+          "         " + common + os.path.join(B_path, 'lambda') + "\n",
+          "         " + common + os.path.join(B_path, 'F') + "\n",
+          "         " + common + B_path + "\n",
+          "         " + common + os.path.join(G_path, 'pi') + "\n",
+          "         " + common + os.path.join(G_path, 'rho') + "\n",
+          "         " + common + os.path.join(G_path, 'tau') + "\n",
+          "         " + common + G_path + "\n",
+          "         " + common + os.path.join(H_path, 'chi') + "\n",
+          "         " + common + os.path.join(H_path, 'omega') + "\n",
+          "         " + common + os.path.join(H_path, 'psi') + "\n",
+          "         " + common + H_path + "\n",
+          "         " + common + os.path.join(D_path, 'gamma') + "\n",
+          "        *" + common + D_path + "\n",
+          "         " + common + os.path.join(A_path, 'mu') + "\n",
+          "         " + common + os.path.join(A_path, 'C') + "\n",
+          "         " + common + A_path + "\n",
+          "         " + common + os.path.join(wc_dir, 'iota') + "\n",
+          "        *" + common + wc_dir  + "\n",
           "Status against revision:      2\n" ])
 
   svntest.actions.run_and_verify_svn(None,
@@ -1225,7 +1226,7 @@ def status_nonrecursive_update(sbox):
                                         '-r', '1', wc_dir)
 
   # Check the remote status of folder A (non-recursively)
-  xout = ["       *        1   " + os.path.join(wc_dir, "A", "mu") + "\n",
+  xout = ["        *        1   " + os.path.join(wc_dir, "A", "mu") + "\n",
           "Status against revision:      2\n" ]
 
   svntest.actions.run_and_verify_svn(None,
@@ -1285,7 +1286,7 @@ def status_depth_local(sbox):
 
   # depth=empty
   expected = svntest.verify.UnorderedOutput(
-                  [" M     %s\n" % A_path])
+                  [" M      %s\n" % A_path])
   svntest.actions.run_and_verify_svn(None,
                                      expected,
                                      [],
@@ -1293,8 +1294,8 @@ def status_depth_local(sbox):
 
   # depth=files
   expected = svntest.verify.UnorderedOutput(
-                  [" M     %s\n" % A_path,
-                   "M      %s\n" % mu_path])
+                  [" M      %s\n" % A_path,
+                   "M       %s\n" % mu_path])
 
   svntest.actions.run_and_verify_svn(None,
                                      expected,
@@ -1303,9 +1304,9 @@ def status_depth_local(sbox):
 
   # depth=immediates
   expected = svntest.verify.UnorderedOutput(
-                  [" M     %s\n" % A_path,
-                   " M     %s\n" % D_path,
-                   "M      %s\n" % mu_path])
+                  [" M      %s\n" % A_path,
+                   " M      %s\n" % D_path,
+                   "M       %s\n" % mu_path])
 
   svntest.actions.run_and_verify_svn(None,
                                      expected,
@@ -1314,10 +1315,10 @@ def status_depth_local(sbox):
 
   # depth=infinity (the default)
   expected = svntest.verify.UnorderedOutput(
-                  [" M     %s\n" % A_path,
-                   " M     %s\n" % D_path,
-                   "M      %s\n" % mu_path,
-                   "M      %s\n" % gamma_path])
+                  [" M      %s\n" % A_path,
+                   " M      %s\n" % D_path,
+                   "M       %s\n" % mu_path,
+                   "M       %s\n" % gamma_path])
 
   svntest.actions.run_and_verify_svn(None,
                                      expected,
@@ -1349,7 +1350,7 @@ def status_depth_update(sbox):
 
   # depth=empty
   expected = svntest.verify.UnorderedOutput(
-                  ["       *        1   %s\n" % A_path,
+                  ["        *        1   %s\n" % A_path,
                    "Status against revision:      3\n"])
 
   svntest.actions.run_and_verify_svn(None,
@@ -1359,8 +1360,8 @@ def status_depth_update(sbox):
 
   # depth=files
   expected = svntest.verify.UnorderedOutput(
-                  ["       *        1   %s\n" % mu_path,
-                   "       *        1   %s\n" % A_path,
+                  ["        *        1   %s\n" % mu_path,
+                   "        *        1   %s\n" % A_path,
                    "Status against revision:      3\n"])
 
   svntest.actions.run_and_verify_svn(None,
@@ -1371,9 +1372,9 @@ def status_depth_update(sbox):
 
   # depth=immediates
   expected = svntest.verify.UnorderedOutput(
-                  ["       *        1   %s\n" % A_path,
-                   "       *        1   %s\n" % D_path,
-                   "       *        1   %s\n" % mu_path,
+                  ["        *        1   %s\n" % A_path,
+                   "        *        1   %s\n" % D_path,
+                   "        *        1   %s\n" % mu_path,
                    "Status against revision:      3\n"])
 
   svntest.actions.run_and_verify_svn(None,
@@ -1384,10 +1385,10 @@ def status_depth_update(sbox):
 
   # depth=infinity (the default)
   expected = svntest.verify.UnorderedOutput(
-                  ["       *        1   %s\n" % A_path,
-                   "       *        1   %s\n" % D_path,
-                   "       *        1   %s\n" % mu_path,
-                   "       *        1   %s\n" % gamma_path,
+                  ["        *        1   %s\n" % A_path,
+                   "        *        1   %s\n" % D_path,
+                   "        *        1   %s\n" % mu_path,
+                   "        *        1   %s\n" % gamma_path,
                    "Status against revision:      3\n"])
 
   svntest.actions.run_and_verify_svn(None,
@@ -1418,12 +1419,12 @@ def status_dash_u_deleted_directories(sbox):
 
   # check status -u of B
   expected = svntest.verify.UnorderedOutput(
-         ["D               1   %s\n" % "B",
-          "D               1   %s\n" % os.path.join("B", "lambda"),
-          "D               1   %s\n" % os.path.join("B", "E"),
-          "D               1   %s\n" % os.path.join("B", "E", "alpha"),
-          "D               1   %s\n" % os.path.join("B", "E", "beta"),
-          "D               1   %s\n" % os.path.join("B", "F"),
+         ["D                1   %s\n" % "B",
+          "D                1   %s\n" % os.path.join("B", "lambda"),
+          "D                1   %s\n" % os.path.join("B", "E"),
+          "D                1   %s\n" % os.path.join("B", "E", "alpha"),
+          "D                1   %s\n" % os.path.join("B", "E", "beta"),
+          "D                1   %s\n" % os.path.join("B", "F"),
           "Status against revision:      1\n" ])
   svntest.actions.run_and_verify_svn(None,
                                      expected,
@@ -1433,12 +1434,12 @@ def status_dash_u_deleted_directories(sbox):
   # again, but now from inside B, should give the same output
   os.chdir("B")
   expected = svntest.verify.UnorderedOutput(
-         ["D               1   %s\n" % ".",
-          "D               1   %s\n" % "lambda",
-          "D               1   %s\n" % "E",
-          "D               1   %s\n" % os.path.join("E", "alpha"),
-          "D               1   %s\n" % os.path.join("E", "beta"),
-          "D               1   %s\n" % "F",
+         ["D                1   %s\n" % ".",
+          "D                1   %s\n" % "lambda",
+          "D                1   %s\n" % "E",
+          "D                1   %s\n" % os.path.join("E", "alpha"),
+          "D                1   %s\n" % os.path.join("E", "beta"),
+          "D                1   %s\n" % "F",
           "Status against revision:      1\n" ])
   svntest.actions.run_and_verify_svn(None,
                                      expected,
@@ -1447,9 +1448,9 @@ def status_dash_u_deleted_directories(sbox):
 
   # check status -u of B/E
   expected = svntest.verify.UnorderedOutput(
-         ["D               1   %s\n" % os.path.join("B", "E"),
-          "D               1   %s\n" % os.path.join("B", "E", "alpha"),
-          "D               1   %s\n" % os.path.join("B", "E", "beta"),
+         ["D                1   %s\n" % os.path.join("B", "E"),
+          "D                1   %s\n" % os.path.join("B", "E", "alpha"),
+          "D                1   %s\n" % os.path.join("B", "E", "beta"),
           "Status against revision:      1\n" ])
 
   os.chdir(was_cwd)
@@ -1476,7 +1477,7 @@ def status_dash_u_type_change(sbox):
   os.remove('iota')
   os.mkdir('iota')
 
-  xout = ["~               1   iota\n",
+  xout = ["~                1   iota\n",
           "Status against revision:      1\n" ]
 
   svntest.actions.run_and_verify_svn(None,
@@ -1489,8 +1490,8 @@ def status_dash_u_type_change(sbox):
   os.mkdir('A')
 
   expected = svntest.verify.UnorderedOutput(
-         ["~               1   iota\n",
-          "~              ?    A\n",
+         ["~                1   iota\n",
+          "~               ?    A\n",
           "Status against revision:      1\n" ])
 
   svntest.actions.run_and_verify_svn(None,
@@ -1498,6 +1499,94 @@ def status_dash_u_type_change(sbox):
                                      [],
                                      "status", "-u")
 
+#----------------------------------------------------------------------
+
+def status_with_tree_conflicts(sbox):
+  "status with tree conflicts"
+
+  # Status messages reflecting tree conflict status.
+  # These tests correspond to use cases 1-3 in
+  # notes/tree-conflicts/use-cases.txt.
+
+  svntest.actions.build_greek_tree_conflicts(sbox)
+  wc_dir = sbox.wc_dir
+  G = os.path.join(wc_dir, 'A', 'D', 'G')
+  pi = os.path.join(G, 'pi')
+  rho = os.path.join(G, 'rho')
+  tau = os.path.join(G, 'tau')
+
+  # check status of G
+  expected = svntest.verify.UnorderedOutput(
+         ["D     C %s\n" % pi,
+          "      >   local delete, incoming edit upon update\n",
+          "A  +  C %s\n" % rho,
+          "      >   local edit, incoming delete upon update\n",
+          "!     C %s\n" % tau,
+          "      >   local delete, incoming delete upon update\n",
+          ])
+
+  svntest.actions.run_and_verify_svn(None,
+                                     expected,
+                                     [],
+                                     "status", G)
+
+  # check status of G, with -v
+  expected = svntest.verify.UnorderedOutput(
+         ["                 2        2 jrandom      %s\n" % G,
+          "D     C          2        2 jrandom      %s\n" % pi,
+          "      >   local delete, incoming edit upon update\n",
+          "A  +  C          -        1 jrandom      %s\n" % rho,
+          "      >   local edit, incoming delete upon update\n",
+          "!     C                                  %s\n" % tau,
+          "      >   local delete, incoming delete upon update\n",
+          ])
+
+
+  svntest.actions.run_and_verify_svn(None,
+                                     expected,
+                                     [],
+                                     "status", "-v", G)
+
+  # check status of G, with -xml
+  exit_code, output, error = svntest.main.run_svn(None, 'status', G, '--xml',
+                                                  '-v')
+
+  should_be_victim = {
+    G:   False,
+    pi:  True,
+    rho: True,
+    tau: True,
+    }
+
+  real_entry_count = 0
+  output_str = r"".join(output)
+  # skip the first string, which contains only 'status' and 'target' elements
+  entries = output_str.split("<entry")[1:]
+
+  for entry in entries:
+    # get the entry's path
+    m = re.search('path="([^"]+)"', entry)
+    if m:
+      real_entry_count += 1
+      path = m.group(1)
+      # check if the path should be a victim
+      m = re.search('tree-conflicted="true"', entry)
+      if (m is None) and should_be_victim[path]:
+        print("ERROR: expected '%s' to be a tree conflict victim." % path)
+        print("ACTUAL STATUS OUTPUT:")
+        print(output_str)
+        raise svntest.Failure
+      if m and not should_be_victim[path]:
+        print("ERROR: did NOT expect '%s' to be a tree conflict victim." % path)
+        print("ACTUAL STATUS OUTPUT:")
+        print(output_str)
+        raise svntest.Failure
+
+  if real_entry_count != len(should_be_victim):
+    print("ERROR: 'status --xml' output is incomplete.")
+    raise svntest.Failure
+
+
 ########################################################################
 # Run the tests
 
@@ -1535,6 +1624,7 @@ test_list = [ None,
               status_depth_local,
               status_depth_update,
               status_dash_u_type_change,
+              status_with_tree_conflicts,
              ]
 
 if __name__ == '__main__':
