@@ -30,13 +30,13 @@ def putfile(fname, rpath, uname="", commitmsg=""):
 
   kind = fs.check_path(root, fname)
   if kind == core.svn_node_none:
-    print "file '%s' does not exist, creating..." % fname
+    print("file '%s' does not exist, creating..." % fname)
     fs.make_file(root, fname)
   elif kind == core.svn_node_dir:
-    print "File '%s' is a dir." % fname
+    print("File '%s' is a dir." % fname)
     return
   else:
-    print "Updating file '%s'" % fname
+    print("Updating file '%s'" % fname)
 
   handler, baton = fs.apply_textdelta(root, fname, None, None)
 
@@ -46,10 +46,10 @@ def putfile(fname, rpath, uname="", commitmsg=""):
                                 handler, baton)
 
   newrev = repos.fs_commit_txn(repos_ptr, txn)
-  print "revision: ", newrev
+  print("revision: %s" % newrev)
 
 def usage():
-  print "USAGE: putfile.py [-m commitmsg] [-u username] file repos-path"
+  print("USAGE: putfile.py [-m commitmsg] [-u username] file repos-path")
   sys.exit(1)
 
 def main():
